/*
 * Copyright (C) 2005 The Mir-coders group
 *
 * This file is part of Mir.
 *
 * Mir is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Mir is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Mir; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * In addition, as a special exception, The Mir-coders gives permission to link
 * the code of this program with  any library licensed under the Apache Software License.
 * You must obey the GNU General Public License in all respects for all of the code used
 * other than the above mentioned libraries.  If you modify this file, you may extend this
 * exception to your version of the file, but you are not obligated to do so.
 * If you do not wish to do so, delete this exception statement from your version.
 */
package mir.generator;

import java.io.File;
import java.io.PrintWriter;
import java.util.Map;

import mir.generator.tal.MirExpressionParser;
import mir.generator.tal.TALTemplateEngine;
import mir.generator.tal.interfaces.TALLogger;
import mir.generator.tal.template.Template;
import mir.log.LoggerWrapper;

public class TALGenerator implements Generator {
  private String templateIdentifier;
  private TALGeneratorLibrary library;
  private Interceptor interceptor;

  public TALGenerator(String aTemplate, TALGeneratorLibrary aLibrary, Interceptor anInterceptor) {
    templateIdentifier = aTemplate;
    library = aLibrary;
    interceptor = anInterceptor;
  }

  public void generate(Object anOutputWriter, Map aValues, final LoggerWrapper aLogger) throws GeneratorExc, GeneratorFailure {
    Template template;

    try {
      template = library.engine.loadTemplate(templateIdentifier);

      if (template == null) {
        throw new GeneratorExc("TALGeneratorLibrary: Can't find template " + templateIdentifier);
      }

      template.process(aValues, (PrintWriter) anOutputWriter,
          new TALLogger() {
            public void debug(String anObject, String aMessage) {
              aLogger.debug(anObject + ": " + aMessage);
            }

            public void info(String anObject, String aMessage) {
              aLogger.info(anObject + ": " + aMessage);
            }

            public void warning(String anObject, String aMessage) {
              aLogger.warn(anObject + ": " + aMessage);
            }

            public void error(String anObject, String aMessage) {
              aLogger.error(anObject + ": " + aMessage);
            }

            public void fatal(String anObject, String aMessage) {
              aLogger.fatal(anObject + ": " + aMessage);
            }
          },
          library.engine);
        }
    catch (Throwable t) {
      ((PrintWriter) anOutputWriter).println("ERROR: " + t.toString());

      throw new GeneratorFailure(t);
    }
  }

  public static class TALGeneratorLibrary implements Library {
    private TALTemplateEngine engine;

    public TALGeneratorLibrary(File aTemplateRoot) throws GeneratorExc, GeneratorFailure {
      engine = new TALTemplateEngine(new MirExpressionParser(), aTemplateRoot);
    }

    public Generator makeGenerator(String anIdentifier, Interceptor anInterceptor) throws GeneratorExc, GeneratorFailure {
      return new TALGenerator(anIdentifier, this, anInterceptor);
    }
  }

  public static class TALGeneratorLibraryFactory implements LibraryFactory {
    private File basePath;

    public TALGeneratorLibraryFactory(File aBasePath) {
      basePath = aBasePath;
    }

    public Library makeLibrary(String anInitializationString) throws GeneratorExc, GeneratorFailure {
      return new TALGeneratorLibrary(new File(basePath, anInitializationString));
    }
  }
}
