/*
 * Copyright (C) 2001-2006 The Mir-coders group
 *
 * This file is part of Mir.
 *
 * Mir is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Mir is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Mir; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * In addition, as a special exception, The Mir-coders gives permission to link
 * the code of this program with  any library licensed under the Apache Software License,
 * and distribute linked combinations including the two.  You must obey the
 * GNU General Public License in all respects for all of the code used other than
 * the above mentioned libraries.  If you modify this file, you may extend this
 * exception to your version of the file, but you are not obligated to do so.
 * If you do not wish to do so, delete this exception statement from your version.
 */
package mir.servlet;

import mir.log.LoggerWrapper;
import mir.config.MirPropertiesConfiguration;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;
import java.lang.reflect.Method;
import java.lang.reflect.InvocationTargetException;
import java.util.Locale;

public abstract class ServletModule {
  private LoggerWrapper logger = new LoggerWrapper("ServletModule." + getName());
  private MirPropertiesConfiguration configuration = MirPropertiesConfiguration.instance();
  private final Locale fallbackLocale = new Locale(
      getConfiguration().getString("Mir.Admin.FallbackLanguage", "en"), "");

  protected ServletModule() {
  }

  /**
   * Return the name of this module
   */
  protected String getName() {
    return getClass().getName().substring("mircoders.servlet.ServletModule".length());
  }

  /**
   * Return the logger for this module
   */
  protected LoggerWrapper getLogger() {
    return logger;
  }

  /**
   * Return the global mir configuration
   */
  protected MirPropertiesConfiguration getConfiguration() {
    return configuration;
  }

  /**
   * signature of request handling methods
   */
  private static final Class[] HANDLER_METHOD_SIGNATURE = {
      HttpServletRequest.class, HttpServletResponse.class
  };

  protected void defaultAction(HttpServletRequest aRequest,
                               HttpServletResponse aResponse)
      throws ServletModuleExc, ServletModuleFailure, ServletModuleUserExc {
    throw new ServletModuleExc("default action not defined for module " + getName());
  }

  /**
   * get the locale either from the session or the accept-language header ot the request
   * this supersedes getLanguage for the new i18n
   */
  protected Locale getFallbackLocale(HttpServletRequest aRequest) {
    return fallbackLocale;
  }

  public Locale[] getLocales(HttpServletRequest aRequest) {
    return new Locale[] { getLocale(aRequest), fallbackLocale };
  }

  /**
   * Return the locale either from the session or the accept-language header ot the request
   * this supersedes getLanguage for the new i18n
   */
  public static Locale getLocale(HttpServletRequest aRequest) {
    Locale locale = null;
    HttpSession session = aRequest.getSession(false);
    if (session != null) {
      // session can be null in case of logout
      locale = (Locale) session.getAttribute("locale");
    }
    // if there is nothing in the session get it fron the accept-language
    if (locale == null) {
      locale = aRequest.getLocale();
    }
    return locale;
  }

  public void handleRequest(HttpServletRequest aRequest,
                            HttpServletResponse aResponse) throws ServletModuleExc, ServletModuleFailure {
    // look for requested method's name in the "do" http request param,
    // if not present, use default action
    String handlerName = aRequest.getParameter("do");
    getLogger().debug("ServletModuleDispatch: " + getClass().getName() + "." + handlerName);

    if (handlerName ==null) {
      handlerName = "defaultAction";
    }

    Method method;
    try {
      method = getClass().getMethod(handlerName, HANDLER_METHOD_SIGNATURE);
    }
    catch (NoSuchMethodException e) {
      throw new ServletModuleFailure("No handler found", e);
    }

    // ok, we have the method's name, now call it
    try {
        method.invoke(this, new Object[] { aRequest,aResponse });
    }
    catch (InvocationTargetException e) {
      getLogger().error("Error while dispatching: " + e.getTargetException().getMessage(),
          e.getTargetException());

      throw new ServletModuleFailure(e.getTargetException().getMessage(), e.getTargetException());
    }
    catch (IllegalAccessException e) {
      throw new ServletModuleFailure(e);
    }
  }

}
